// =============================================================================================================================//
// Константы для MQTT                                                                                                           //
// =============================================================================================================================//
// Автор: Александр Разживин, kotyara12@yandex.ru, http://ravsoft2004.narod.ru                                                  //
// =============================================================================================================================//

#ifndef rConstMQTT_h
#define rConstMQTT_h 

#include "Arduino.h"
#include <TimeLib.h>

// Топики
extern const char* mqttDiv;
extern const char* mqttPlus;
extern const char* mqttSettings;
extern const char* mqttGateway;
extern const char* mqttGatewayAct;
extern const char* mqttGatewayName;
extern const char* mqttSyncStatus;
extern const char* mqttSyncMode;
extern const char* mqttSyncRequest;
extern const char* mqttSyncOk;
extern const char* mqttSyncWill;
extern const char* mqttTrue;
extern const char* mqttFalse;
extern const char* mqttOn;
extern const char* mqttOff;

extern const char* mqttPubInterval;
extern const char* mqttThingSpeak;

extern const char* mqttCommands;
extern const char* mqttCmdReset;

extern const char* mqttSensorName;
extern const char* mqttSensorState;
extern const char* mqttSensorType;
extern const char* mqttSensorTimeI;
extern const char* mqttSensorTimeL;
extern const char* mqttSensorTimeS;

extern const char* mqttPressure;
extern const char* mqttPressurePa;
extern const char* mqttPressureHg;
extern const char* mqttTemperature;
extern const char* mqttHumidity;
extern const char* mqttHeatIndex;
extern const char* mqttDewPoint;
extern const char* mqttRSSI;

extern const char* mqttDeviceInfo;
extern const char* mqttChipId;
extern const char* mqttFlashChipId;
extern const char* mqttCpuFreqMHz;
extern const char* mqttFlashChipRealSize;
extern const char* mqttFlashChipSpeed;
extern const char* mqttIp;
extern const char* mqttRSSI;
extern const char* mqttTime;
extern const char* mqttTimeS;
extern const char* mqttTimeL;
extern const char* mqttPowerVoltage;
extern const char* mqttHeapFree;
extern const char* mqttHeapFrag;

// Сенсоры
extern const char* stateOK;
extern const char* stateDisconnected;

extern const char* sensorNone;
extern const char* sensorBme280;
extern const char* sensorDht11;
extern const char* sensorDht22;
extern const char* sensorRht03;
extern const char* sensorAm2302;

extern const double prsPa2mmHg;

struct sensorValue {
  String topic;
  String value;
};

const byte maxValues = 8;

class sensorData {
  public:
    sensorData(const String _name, const String _type, const String _state, time_t _time) {
      sensorName = _name;
      sensorType = _type;
      sensorState = _state;
      sensorTime = _time;
      countValues = 0;
      for (byte i=0;i<maxValues;i++) 
      {
        arrayValues[i].topic = "";
        arrayValues[i].value = "";
      };
    };

    void addValue(const String _topic, const String _value) {
      if (countValues < maxValues) {
        arrayValues[countValues].topic = _topic;
        arrayValues[countValues].value = _value;
        countValues++;
      };
    };

    void addTime() {
      sensorTime = now();
    }

    String sensorName;
    String sensorType;
    String sensorState;
    time_t sensorTime;
    byte countValues;
    sensorValue arrayValues[maxValues];
};

#endif
