// =============================================================================================================================//
// Расширения модуля <EEPROM.h> для работы с различными типами данных                                                           //
// Прототипы надыбаны где-то "в этих ваших интернетах"...                                                                       //
// =============================================================================================================================//

#include "Arduino.h"
#include "rEEPROM.h" 

void EEPROM_bool_write(int addr, bool value) {
  byte raw = 0;
  if (value) { raw = 1; };
  EEPROM.write(addr, raw);
  EEPROM.commit();
}

bool EEPROM_bool_read(int addr, bool def_value) {
  byte raw = EEPROM.read(addr);
  if (raw == 0xFF) {
    EEPROM_bool_write(addr, def_value);
    return def_value;
  }
  return raw > 0;
}

void EEPROM_uint8_write(int addr, uint8_t value) {
  EEPROM.write(addr, value);
  EEPROM.commit();
}

uint8_t EEPROM_uint8_read(int addr, uint8_t def_value) {
  uint8_t raw = EEPROM.read(addr);
  if (raw == 0xFF) {
    EEPROM_uint8_write(addr, def_value);
    return def_value;
  }
  return raw;
}

void EEPROM_uint16_write(int addr, uint16_t value) {
  byte raw[2];
  (uint16_t&)raw = value;
  for(byte i = 0; i < 2; i++) EEPROM.write(addr+i, raw[i]);
  EEPROM.commit();
}

uint16_t EEPROM_uint16_read(int addr, uint16_t def_value) {   
  byte raw[2];
  for(byte i = 0; i < 2; i++) raw[i] = EEPROM.read(addr+i);
  if (raw[0] == 0xFF) {
    EEPROM_uint16_write(addr, def_value);
    return def_value;
  }
  else {
    uint16_t &value = (uint16_t&)raw;
    return value;
  };
}

void EEPROM_uint32_write(int addr, uint32_t value) {
  byte raw[4];
  (uint32_t&)raw = value;
  for(byte i = 0; i < 4; i++) EEPROM.write(addr+i, raw[i]);
  EEPROM.commit();
}

uint32_t EEPROM_uint32_read(int addr, uint32_t def_value) {   
  byte raw[4];
  for(byte i = 0; i < 4; i++) raw[i] = EEPROM.read(addr+i);
  if (raw[0] == 0xFF) {
    EEPROM_uint32_write(addr, def_value);
    return def_value;
  }
  else {
    uint32_t &value = (uint32_t&)raw;
    return value;
  };
}

void EEPROM_float_write(int addr, float value) {
  byte raw[4];
  (float&)raw = value;
  for(byte i = 0; i < 4; i++) EEPROM.write(addr+i, raw[i]);
  EEPROM.commit();
}

float EEPROM_float_read(int addr, float def_value) {   
  byte raw[4];
  for(byte i = 0; i < 4; i++) raw[i] = EEPROM.read(addr+i);
  if (raw[0] == 0xFF) {
    EEPROM_float_write(addr, def_value);
    return def_value;
  }
  else {
    float &value = (float&)raw;
    return value;
  };
}
