// =============================================================================================================================//
// Модуль для работы с MQTT-серверами, "обертка" для PubSubClient                                                               //
// =============================================================================================================================//
// Автор: Александр Разживин, kotyara12@yandex.ru, http://ravsoft2004.narod.ru                                                  //
// =============================================================================================================================//

#ifndef rMQTT_h
#define rMQTT_h 

#if defined(ESP8266) || defined(ESP32)
#include <functional>
#include <pgmspace.h>
#endif 

#include "Arduino.h"
#include "rLed.h"
#include "rConstMQTT.h"

typedef std::function<bool(const String topicName, const String topicValue)> t_cbSubSettings;
typedef std::function<void()> t_cbPubSensors;

void   mqttBegin(rLed &led, const char* sServer, const uint16_t iPort, const char* sUser, const char* sPass, const char* sSite, const char* sDeviceId, const bool bStartSlash);

void   mqttSetCallbackSensors(t_cbPubSensors cb);
void   mqttSetCallbackSub(t_cbSubSettings cb);

void   mqttReadSettings();

String mqttGetSite();
String mqttGetPrefix();
String mqttGetTopic(const String topic1);
String mqttGetTopic(const String topic1, const String topic2);
String mqttGetTopic(const String topic1, const String topic2, const String topic3);
String mqttGetTopic(const String topic1, const String topic2, const String topic3, const String topic4);
String mqttGetGlobalTopic(const String topic1, const String topic2);
String mqttGetGlobalTopic(const String topic1, const String topic2, const String topic3);
String mqttGetGlobalTopic(const String topic1, const String topic2, const String topic3, const String topic4);

bool   mqttConnect();
bool   mqttForceReconnect();

void   tsPublish(const String request);
bool   mqttPublish(const String topic, const String payload, const boolean retain, const byte qos);

void   mqttSubscribe(const String topic);
void   mqttUnsubscribe(const String topic);

void   mqttPublishSensor(const String snsTopic, const String itemTopic, const String itemValue);
void   mqttPublishSensor(const String snsTopic, const String itemTopic, const char* tsKeyAPI, const byte tsFieldNo, const String itemValue);
void   mqttPublishSensorData(const String snsTopic, const char* tsKeyAPI, const sensorData &snsData);
void   mqttPublishDeviceInfo();
void   mqttPublishSensors();

void   mqttLoop();

#endif
