// =============================================================================================================================//
// Расширения модуля <TimeLib.h> для синхронизации времени через NTP сервера                                                    //
// Прототипы надыбаны где-то "в этих ваших интернетах"...                                                                       //
// =============================================================================================================================//

#include "Arduino.h"
#include "TimeLib.h"
#include "rNTP.h"

int            ntpTimeZone         = 0;

const char*    ntpServer1          = "ntp1.stratum2.ru";
const char*    ntpServer2          = "ntp2.stratum2.ru";
const char*    ntpServer3          = "ntp1.stratum1.ru";
const char*    ntpServer4          = "pool.ntp.org";
const char*    ntpServer5          = "time.nist.gov";

const int      NTP_PACKET_SIZE = 48;  //  NTP-время – в первых 48 байтах сообщения
byte           ntpBuffer[NTP_PACKET_SIZE];  //  буфер для хранения входящих и исходящих пакетов 

WiFiUDP udpNtp;

String timeGetDigits(int digits) 
{
  if (digits < 10) {
    return String("0") + String(digits);
  }
  else {
    return String(digits);
  }
} 

String timeTimeShortString(time_t t) {
  return timeGetDigits(hour(t)) + ":" + timeGetDigits(minute(t));
}

String timeTimeShortString() {
  return timeGetDigits(hour()) + ":" + timeGetDigits(minute());
}

String timeTimeString(time_t t) {
  return timeGetDigits(hour(t)) + ":" + timeGetDigits(minute(t)) + ":" + timeGetDigits(second(t));
}

String timeTimeString() {
  return timeGetDigits(hour()) + ":" + timeGetDigits(minute()) + ":" + timeGetDigits(second());
}

String timeDateString(time_t t) {
  return timeGetDigits(day(t)) + "." + timeGetDigits(month(t)) + "." + String(year(t));
}

String timeDateString() {
  return timeGetDigits(day()) + "." + timeGetDigits(month()) + "." + String(year());
}

String timeDateTimeString(time_t t) {
  return timeDateString(t) + " " + timeTimeString(t);
}

String timeDateTimeString() {
  return timeDateString() + " " + timeTimeString();
}

// =============================================================================================================================
// == NTP private ==============================================================================================================
// =============================================================================================================================

// Отправляем NTP-запрос серверу времени по указанному адресу
void ntpSendRequest(IPAddress &address) {
  // задаем все байты в буфере на «0»: 
  memset(ntpBuffer, 0, NTP_PACKET_SIZE);
  // инициализируем значения для создания NTP-запроса
  // (подробнее о пакетах смотрите по ссылке выше) 
  ntpBuffer[0] = 0b11100011;   // LI (от «leap indicator», т.е. «индикатор перехода»), версия, режим работы 
  ntpBuffer[1] = 0;            // слой (или тип часов) 
  ntpBuffer[2] = 6;            // интервал запросов 
  ntpBuffer[3] = 0xEC;         // точность 
  // 8 байтов с нулями, обозначающие базовую задержку и базовую дисперсию: 
  ntpBuffer[12] = 49;
  ntpBuffer[13] = 0x4E;
  ntpBuffer[14] = 49;
  ntpBuffer[15] = 52;
  // После заполнения всех указанных полей
  // вы сможете отправлять пакет с запросом о временной метке:      
  udpNtp.beginPacket(address, 123); // NTP-запросы к порту 123
  udpNtp.write(ntpBuffer, NTP_PACKET_SIZE);
  udpNtp.endPacket();
}

// Получение NTP времени для конкретного сервера
time_t ntpSyncTimeServer(const char* ntpServerName) {
  IPAddress ntpServerIP; // IP-адрес NTP-сервера

  // Отбраковываем все пакеты, полученные ранее 
  // while (udpNtp.parsePacket() > 0); 
  udpNtp.flush(); 

  // Преобразовываем имя сервера в IP адрес
  WiFi.hostByName(ntpServerName, ntpServerIP);

  // Передача NTP-запроса
  Serial.println(F("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"));
  Serial.print(F("NTP :: Transmit NTP request to '"));  
  Serial.print(ntpServerName);
  Serial.print("' / '");
  Serial.print(ntpServerIP);
  Serial.print("': ");
  ntpSendRequest(ntpServerIP);

  // Получение NTP-ответа
  uint32_t beginWait = millis();
  while (millis() - beginWait < 1500) {
    int size = udpNtp.parsePacket();
    if (size >= NTP_PACKET_SIZE) {
      Serial.println("ok");  
      Serial.println(F("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"));
      Serial.println();
      // Считываем пакет в буфер 
      udpNtp.read(ntpBuffer, NTP_PACKET_SIZE);  
      // Очищаем буфер UTP
      udpNtp.flush(); 
      // Конвертируем 4 байта (начиная с позиции 40) в длинное целое число
      unsigned long secsSince1900;
      secsSince1900 =  (unsigned long)ntpBuffer[40] << 24;
      secsSince1900 |= (unsigned long)ntpBuffer[41] << 16;
      secsSince1900 |= (unsigned long)ntpBuffer[42] << 8;
      secsSince1900 |= (unsigned long)ntpBuffer[43];
      return secsSince1900 - 2208988800UL + ntpTimeZone * SECS_PER_HOUR;
    };
  };

  // Нет NTP-ответа :(
  Serial.println("no NTP response :-(");  
  Serial.println(F("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"));
  Serial.println();
  // Eсли время получить не удалось, возвращаем «0» 
  return 0; 
}

// Получение NTP времени по списку серверов
time_t ntpSyncTime() {
  time_t result = ntpSyncTimeServer(ntpServer1);  
  if (result == 0) {
    delay(500);
    result = ntpSyncTimeServer(ntpServer2);  
  };
  if (result == 0) {
    delay(500);
    result = ntpSyncTimeServer(ntpServer3);  
  };
  if (result == 0) {
    delay(500);
    result = ntpSyncTimeServer(ntpServer4);  
  };
  if (result == 0) {
    delay(500);
    result = ntpSyncTimeServer(ntpServer5);  
  };
  return result;
}

// =============================================================================================================================
// == NTP public ===============================================================================================================
// =============================================================================================================================

// Инициализация NTP-синхронизации
void ntpTimeSyncInit(unsigned int iLocalPort, const int iTimeZone) {
  udpNtp.begin(iLocalPort);
  ntpTimeZone = iTimeZone;
}

// Запуск NTP-синхронизации
void ntpTimeSyncRun(const int iSyncInterval) {
  setSyncProvider(ntpSyncTime);
  setSyncInterval(iSyncInterval);

  Serial.print(F("NTP :: Current time: "));
  Serial.println(timeDateTimeString());
}
