
// ==============================================================================//
// Телеметр (персональная метеостанция) на ESP8266 NodeMcu V3                    //
// ==============================================================================//
// Используется 4 сенсора:                                                       // 
// 1. BME280  - улица ( канал: https://thingspeak.com/channels/654121 )          //
// 2. DHT22   - гараж                                                            //
// 3. DHT22   - погреб 1                                                         //
// 4. DHT22   - погреб 2 (по этому датчику включается подогрев)                  //
// ==============================================================================//
// Данные передаются сразу на 2 сервера: MQTT cloudmqtt.com и thingspeak.com     //
// ==============================================================================//
// Автор: Александр Разживин, kotyara12@yandex.ru, http://ravsoft2004.narod.ru   //
// ==============================================================================//

#include <Wire.h>
#include <Adafruit_BME280.h>
#include <Adafruit_Sensor.h>
#include <DHTesp.h>

#include "rWiFi.h"
#include "rEEPROM.h"
#include "rLed.h"
#include "rNTP.h"
#include "rConstMQTT.h"
#include "rMQTT.h"

// =============================================================================================================================
// == Настройки для конкретного исполнения =====================================================================================
// =============================================================================================================================

// Выводы, к которым подкючена периферия
// #define pinSCL D1
// #define pinSCA D2
#define pinLED    D3
#define pinLEDINT D4
#define pinSENS2  D5
#define pinSENS3  D7
#define pinSENS4  D6
#define pinRELAY  D8

// Параметры сетей WiFi
const char*    wifi1_ssid          = "************";      // Имя WiFi сети 1
const char*    wifi1_pass          = "************";      // Пароль WiFi сети 1
const char*    wifi2_ssid          = "************";      // Имя WiFi сети 2
const char*    wifi2_pass          = "************";      // Пароль WiFi сети 2
const char*    wifi3_ssid          = "************";      // Имя WiFi сети 3
const char*    wifi3_pass          = "************";      // Пароль WiFi сети 4
String         ssidWiFiLast        = "";                  // Имя последней удачной WiFi сети
String         passWiFiLast        = "";                  // Пароль последней удачной WiFi сети

// Параметры локального сервера MQTT
const char*    mqttServer          = "mqtt.flespi.io";    // Имя сервера MQTT
const int      mqttPort            = 1883;                // Порт для подключения к серверу MQTT
const char*    mqttUser            = "*****************"; // Логин от сервера MQTT
const char*    mqttPass            = "";                  // Пароль от сервера MQTT
const char*    mqttSite            = "village";           // Префикс топиков (местоположение)
const char*    mqttDeviceId        = "garage";            // Идентификатор клиента MQTT
const bool     mqttStartSlash      = false;               // Топики всегда начинать с "/"

// Топики сенсоров и устройств
const char*    mqttSensor1         = "outdoor";           // Улица
const char*    mqttSensor2         = "garage";            // Гараж 
const char*    mqttSensor3         = "cellar_up";         // Погреб 1
const char*    mqttSensor4         = "cellar_down";       // Погреб 2

const char*    nameSensor1         = "Дача: улица";       // Улица
const char*    nameSensor2         = "Дача: гараж";       // Гараж 
const char*    nameSensor3         = "Дача: погреб 1";    // Погреб 1
const char*    nameSensor4         = "Дача: погреб 2";    // Погреб 2

const char*    tkeySensor1         = "****************";  // Улица
const char*    tkeySensor2         = "****************";  // Гараж
const char*    tkeySensor3         = "****************";  // Погреб 1
const char*    tkeySensor4         = "****************";  // Погреб 2

const char*    mqttHeater          = "heater";            // Нагреватель
const char*    mqttHeaterMode      = "heater_mode";       // Режим управления нагревателем
const char*    mqttHeaterMin       = "heater_min";        // Минимальная температура на автоматического режима
const char*    mqttHeaterMax       = "heater_max";        // Максимальная температура на автоматического режима

const char*    mqttHeaterDOff      = "0";                 // Режим управления нагревателем "отключено"
const char*    mqttHeaterDOn       = "1";                 // Режим управления нагревателем "включено"
const char*    mqttHeaterDAuto     = "2";                 // Режим управления нагревателем "автоматически"
const char*    mqttHeaterSOff      = "отключено";         // Режим управления нагревателем "отключено"
const char*    mqttHeaterSOn       = "включено";          // Режим управления нагревателем "включено"
const char*    mqttHeaterSAuto     = "автоматически";     // Режим управления нагревателем "автоматически"

const int16_t  addrHeaterMode      = 32;                  // Режим управления нагревателем
const int16_t  addrHeaterAutoOn    = 33;                  // Температура, при которой включается нагреватель
const int16_t  addrHeaterAutoOff   = 34;                  // Температура, при которой выключается нагреватель
const int16_t  addrCorrTemp1       = 36;                  // Корректировка температуры сенсора 1
const int16_t  addrCorrPress1      = 40;                  // Корректировка давления сенсора 1
const int16_t  addrCorrHum1        = 44;                  // Корректировка влажности сенсора 1
const int16_t  addrCorrTemp2       = 48;                  // Корректировка температуры сенсора 2
const int16_t  addrCorrHum2        = 52;                  // Корректировка влажности сенсора 2
const int16_t  addrCorrTemp3       = 56;                  // Корректировка температуры сенсора 3
const int16_t  addrCorrHum3        = 60;                  // Корректировка влажности сенсора 3
const int16_t  addrCorrTemp4       = 64;                  // Корректировка температуры сенсора 4
const int16_t  addrCorrHum4        = 68;                  // Корректировка влажности сенсора 4

const bool     retHeater           = true;                // Retained для нагревателя
const byte     qosHeater           = 2;                   // QoS для нагревателя

// =============================================================================================================================
// == Рабочие параметры ========================================================================================================
// =============================================================================================================================
int8_t         heaterMode          = 0;                   // Режим управления нагревателем
bool           heaterState         = false;               // Текущий режим нагревателя
int8_t         heaterAutoOn        = 0;                   // Температура, при которой включается нагреватель
int8_t         heaterAutoOff       = 5;                   // Температура, при которой выключается нагреватель
bool           heaterTempReady     = false;               // Признак измерянного состояния температуры
float          heaterTempValue     = 0;                   // Последняя измерянная температура для нагревателя
bool           stateSensor1        = false;               // Статус наружнего сенсора
float          sensCorrTemp1       = 0;                   // Корректировка температуры сенсора 1
float          sensCorrPress1      = 0;                   // Корректировка давления сенсора 1
float          sensCorrHum1        = 0;                   // Корректировка влажности сенсора 1
float          sensCorrTemp2       = 0;                   // Корректировка температуры сенсора 2
float          sensCorrHum2        = 0;                   // Корректировка влажности сенсора 2
float          sensCorrTemp3       = 0;                   // Корректировка температуры сенсора 3
float          sensCorrHum3        = 0;                   // Корректировка влажности сенсора 3
float          sensCorrTemp4       = 0;                   // Корректировка температуры сенсора 4
float          sensCorrHum4        = 0;                   // Корректировка влажности сенсора 4

// rLed led1(pinLEDINT, LOW);
rLed led1(pinLED, HIGH);

Adafruit_BME280 bmeSensor1;
DHTesp dhtSensor2;
DHTesp dhtSensor3;
DHTesp dhtSensor4;

WiFiEventHandler wifiConnectedHandler;
WiFiEventHandler wifiGotIPHandler;
WiFiEventHandler wifiDisconnectedHandler; 

// =============================================================================================================================
// == Инициализация устройства =================================================================================================
// =============================================================================================================================
void setup() 
{
  Serial.begin(115200);
  Serial.println(F("Initialization..."));

  // Настройка портов
  Serial.println(F("Inititailize PINs..."));
  // ADC_MODE(ADC_VCC);
  led1.begin();
  pinMode(pinRELAY, OUTPUT);
  pinMode(pinLEDINT, OUTPUT);
  digitalWrite(pinLEDINT, HIGH);

  // Настраиваем WiFi подключение
  Serial.println(F("Inititailize WiFi connection..."));
  WiFi.disconnect();

  // Настраиваем MQTT подключение
  Serial.println(F("Inititailize MQTT connection..."));
  mqttBegin(led1, mqttServer, mqttPort, mqttUser, mqttPass, mqttSite, mqttDeviceId, mqttStartSlash);
  mqttSetCallbackSensors(pubSensors);
  mqttSetCallbackSub(subSettings);

  // Считываем пераметры из памяти
  Serial.println(F("Read settings from EEPROM..."));
  EEPROM.begin(512); 

  mqttReadSettings();
  
  Serial.print(F("heaterMode = "));
  heaterMode = EEPROM_uint8_read(addrHeaterMode, heaterMode);
  Serial.println(heaterMode);

  Serial.print(F("heaterAutoOn = "));
  heaterAutoOn = EEPROM_uint8_read(addrHeaterAutoOn, heaterAutoOn);
  Serial.println(heaterAutoOn);

  Serial.print(F("heaterAutoOff = "));
  heaterAutoOff = EEPROM_uint8_read(addrHeaterAutoOff, heaterAutoOff);
  Serial.println(heaterAutoOff);

  Serial.print(F("sensCorrTemp1 = "));
  sensCorrTemp1 = EEPROM_float_read(addrCorrTemp1, sensCorrTemp1);
  Serial.println(sensCorrTemp1);

  Serial.print(F("sensCorrPress1 = "));
  sensCorrPress1 = EEPROM_float_read(addrCorrPress1, sensCorrPress1);
  Serial.println(sensCorrPress1);

  Serial.print(F("sensCorrHum1 = "));
  sensCorrHum1 = EEPROM_float_read(addrCorrHum1, sensCorrHum1);
  Serial.println(sensCorrHum1);

  Serial.print(F("sensCorrTemp2 = "));
  sensCorrTemp2 = EEPROM_float_read(addrCorrTemp2, sensCorrTemp2);
  Serial.println(sensCorrTemp2);

  Serial.print(F("sensCorrHum2 = "));
  sensCorrHum2 = EEPROM_float_read(addrCorrHum2, sensCorrHum2);
  Serial.println(sensCorrHum2);

  Serial.print(F("sensCorrTemp3 = "));
  sensCorrTemp3 = EEPROM_float_read(addrCorrTemp3, sensCorrTemp3);
  Serial.println(sensCorrTemp3);

  Serial.print(F("sensCorrHum3 = "));
  sensCorrHum3 = EEPROM_float_read(addrCorrHum3, sensCorrHum3);
  Serial.println(sensCorrHum3);

  Serial.print(F("sensCorrTemp4 = "));
  sensCorrTemp4 = EEPROM_float_read(addrCorrTemp4, sensCorrTemp4);
  Serial.println(sensCorrTemp4);

  Serial.print(F("sensCorrHum4 = "));
  sensCorrHum4 = EEPROM_float_read(addrCorrHum4, sensCorrHum4);
  Serial.println(sensCorrHum4);

  // Настраиваем NTP
  Serial.println(F("Inititailize NTP..."));
  ntpTimeSyncInit();

  // Настраиваем I2C
  Serial.println(F("Inititailize sensors..."));
  Wire.begin();
  Wire.setClock(100000); 

  // Подключаем уличный датчик BME280
  stateSensor1 = bmeSensor1.begin(0x76);
  if (stateSensor1) {
    Serial.println(F("Outdoor BME280 sensor connected"));
  }
  else {
    Serial.println(F("Could not find a valid BME280 sensor, check wiring!"));
  };

  // Подключаем датчики температуры DHT
  dhtSensor2.setup(pinSENS2, DHTesp::AM2302);
  dhtSensor3.setup(pinSENS3, DHTesp::AM2302);
  dhtSensor4.setup(pinSENS4, DHTesp::AM2302);
  
  // Инициализируем нагреватель
  Serial.println("Inititailize heater...");
  heaterTempReady = false;
  heaterTempValue = 0;
  heaterToggle(false);
    
  Serial.println(F("Initialization complete"));
  Serial.println();
}

// =============================================================================================================================
// == Главный цикл =============================================================================================================
// =============================================================================================================================
void loop() 
{
  if (wifiCheckConnect()) {
    mqttLoop();
  };

  delay(100);
}

// =============================================================================================================================
// == Подключение к WiFi =======================================================================================================
// =============================================================================================================================
void onWiFiConnected(const WiFiEventStationModeConnected& evt) 
{
  Serial.print(F("WiFi :: Connected to ssid '"));
  Serial.print(evt.ssid);
  Serial.print(F("', channel "));
  Serial.print(evt.channel);
  Serial.print(F(", RSSI: "));
  Serial.println(WiFi.RSSI());
}

void onWiFiGotIP(const WiFiEventStationModeGotIP& evt)
{
  Serial.print(F("WiFi :: Local IP address: "));
  Serial.println(evt.ip);
}

void onWiFiDisconnected(const WiFiEventStationModeDisconnected& evt) 
{
  Serial.println();
  Serial.println(F("*****************************************************************************************"));
  Serial.print(F("WiFi: Connection lost, reason: "));
  Serial.print(wifiReasonText(evt.reason));
  Serial.println(F(" !"));
  Serial.println(F("*****************************************************************************************"));
} 

bool _wifiConnect(const char* ssid, const char* pass) 
{
  Serial.print(F("WiFi :: Connect to network '"));
  Serial.print(ssid);
  Serial.println(F("'..."));

  int tryCnt = 100;
  
  WiFi.disconnect();
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, pass);
  
  wifiConnectedHandler = WiFi.onStationModeConnected(&onWiFiConnected);
  wifiGotIPHandler = WiFi.onStationModeGotIP(&onWiFiGotIP);
  wifiDisconnectedHandler = nullptr;
  
  wl_status_t wifiStatus = WiFi.status();
  while ((tryCnt > 0) & (wifiStatus != WL_CONNECTED)) { 
    tryCnt--;
    delay(250); 
    led1.ledToggle();
    wifiStatus = WiFi.status();
    yield();
  };

  if (wifiStatus == WL_CONNECTED) {
    wifiDisconnectedHandler = WiFi.onStationModeDisconnected(&onWiFiDisconnected);
  }
  else {
    Serial.print(F("WiFi :: Connection failed: "));
    Serial.println(wifiStatusText(wifiStatus));
  };

  Serial.println(F("*****************************************************************************************"));
  
  return (wifiStatus == WL_CONNECTED);
}

bool wifiConnect(const char* ssid, const char* pass) 
{
  bool result = _wifiConnect(ssid, pass);
  if (result) {
    Serial.println();
    delay(500);
    ntpTimeSyncRun();
    led1.ledOff();
  };
  return result;
}

bool wifiConnectMulti() 
{
  if (wifiConnect(wifi1_ssid, wifi1_pass)) {
    ssidWiFiLast = wifi1_ssid; 
    passWiFiLast = wifi1_pass;
    return true;
  };

  if (wifiConnect(wifi2_ssid, wifi2_pass)) {
    ssidWiFiLast = wifi2_ssid; 
    passWiFiLast = wifi2_pass;
    return true;
  };

  if (wifiConnect(wifi3_ssid, wifi3_pass)) {
    ssidWiFiLast = wifi3_ssid; 
    passWiFiLast = wifi3_pass;
    return true;
  };

  ssidWiFiLast = "";
  passWiFiLast = "";
  return false;
}

bool wifiCheckConnect() 
{
  // Проверяем текущий статус
  wl_status_t wifiStatus = WiFi.status();
  bool result = (wifiStatus == WL_CONNECTED);
  if (!result) {
    Serial.println(F("*****************************************************************************************"));
    Serial.print("WiFi :: Network status: ");
    Serial.println(wifiStatusText(wifiStatus));
    Serial.println(F("*****************************************************************************************"));
  };
  
  // Пробуем подключиться к последней удачной сети
  if ((!result) && (ssidWiFiLast != "")) {
    result = wifiConnect(ssidWiFiLast.c_str(), passWiFiLast.c_str());  
  };
   
  // Иначе проверяем все сети
  if (!result) {
    result = wifiConnectMulti();
  };

  return result;
}

// =============================================================================================================================
// == Управление нагревателем ==================================================================================================
// =============================================================================================================================
void relayOn() {
  digitalWrite(pinRELAY, HIGH);
}

void relayOff() {
  digitalWrite(pinRELAY, LOW);
}

void heaterToggle(const bool newState) {
  // Переключаем реле
  heaterState = newState;
  
  if (heaterState == true) {
    relayOn();
    Serial.println(F("HEATER :: turn ON!"));
  }
  else {
    relayOff();
    Serial.println(F("HEATER :: turn OFF!"));
  };
}

void heaterControl() {
  switch (heaterMode) {
    case 0: 
      if (heaterState) heaterToggle(false);
      break;
    case 1:
      if (!heaterState) heaterToggle(true);
      break;
    case 2:
      if (heaterTempReady) {
        if (!heaterState && (heaterTempValue <= heaterAutoOn)) heaterToggle(true);
        if (heaterState && (heaterTempValue >= heaterAutoOff)) heaterToggle(false);
      };
      break;
  };
}

// =============================================================================================================================
// == MQTT callbacks ===========================================================================================================
// =============================================================================================================================
void pubSensorBME(Adafruit_BME280 bme, const bool snsState, const char* snsName, const char* snsTopic, const char* snsKey,
  const float corrTemp, const float corrPress, const float corrHum) 
{
  sensorData snsData(snsName, sensorNone, stateDisconnected, now());

  if (snsState) {
    // Считываем параметры из датчика
    float pressure = bme.readPressure() + corrPress;
    float humidity = bme.readHumidity() + corrHum;
    float temperature = bme.readTemperature() + corrTemp;
    float heatindex = dhtSensor2.computeHeatIndex(temperature, humidity, false);
    float dewpoint = dhtSensor2.computeDewPoint(temperature, humidity, false);

    if (temperature >= -50.0) {
      snsData.sensorType = sensorBme280;
      snsData.sensorState = stateOK;

      // Добавляем параметры в массив
      snsData.addValue(mqttPressurePa, String(pressure));
      snsData.addValue(mqttPressureHg, String(pressure / prsPa2mmHg));
      snsData.addValue(mqttTemperature, String(temperature));
      snsData.addValue(mqttHumidity, String(humidity));
      snsData.addValue(mqttHeatIndex, String(heatindex));
      snsData.addValue(mqttDewPoint, String(dewpoint));
    };
  };

  // Отправляем данные
  mqttPublishSensorData(snsTopic, snsKey, snsData);
}

void pubSensorDHT(DHTesp dht, const char* snsName, const char* snsTopic, const char* snsKey, const bool isHeater,
  const float corrTemp, const float corrHum) 
{
  sensorData snsData(snsName, sensorNone, stateDisconnected, now());

  // Считываем параметры из датчика
  TempAndHumidity newValues = dht.getTempAndHumidity();
  
  // Если датчик выдал ошибку ERROR_TIMEOUT, ждем и пробуем еще раз
  if (dht.getStatus() == DHTesp::ERROR_TIMEOUT) { 
    delay(dht.getMinimumSamplingPeriod()); 
    newValues = dht.getTempAndHumidity();
  };

  snsData.sensorState = dht.getStatusString();
  if (dht.getStatus() != DHTesp::ERROR_TIMEOUT) { 
    switch (dht.getModel()) {
      case DHTesp::DHT11: 
        snsData.sensorType = sensorDht11;
        break;
      case DHTesp::DHT22: 
        snsData.sensorType = sensorDht22;
        break;
      case DHTesp::AM2302: 
        snsData.sensorType = sensorAm2302;
        break;
      case DHTesp::RHT03: 
        snsData.sensorType = sensorRht03;
        break;
      defailt:  
        snsData.sensorType = sensorNone;
    };
  };
  
  if (dht.getStatus() == DHTesp::ERROR_NONE) {
    newValues.temperature = newValues.temperature + corrTemp;
    newValues.humidity = newValues.humidity + corrHum;
    float heatindex = dht.computeHeatIndex(newValues.temperature, newValues.humidity, false);
    float dewpoint = dht.computeDewPoint(newValues.temperature, newValues.humidity, false);

    snsData.addValue(mqttTemperature, String(newValues.temperature));
    snsData.addValue(mqttHumidity, String(newValues.humidity));
    snsData.addValue(mqttHeatIndex, String(heatindex));
    snsData.addValue(mqttDewPoint, String(dewpoint));

    if (isHeater) {
      heaterTempValue = newValues.temperature;
      heaterTempReady = true;
      heaterControl();
    };
  }
  else {
    if (isHeater) {
      heaterTempReady = false;
    };
  };

  // Отправляем данные
  mqttPublishSensorData(snsTopic, snsKey, snsData);
}

void pubHeaterState() 
{
  mqttPublishSensor(mqttSensor4, mqttHeater, tkeySensor4, 5, String(heaterState));  
};

void pubSensors() {
  pubSensorBME(bmeSensor1, stateSensor1, nameSensor1, mqttSensor1, tkeySensor1, sensCorrTemp1, sensCorrPress1, sensCorrHum1);  
  pubSensorDHT(dhtSensor2, nameSensor2, mqttSensor2, tkeySensor2, false, sensCorrTemp2, sensCorrHum2);
  pubSensorDHT(dhtSensor3, nameSensor3, mqttSensor3, tkeySensor3, false, sensCorrTemp3, sensCorrHum3);
  pubSensorDHT(dhtSensor4, nameSensor4, mqttSensor4, tkeySensor4, true, sensCorrTemp4, sensCorrHum4);
  pubHeaterState();
}

bool subSettings(const String topicName, const String topicValue) {
  // Переключение режима нагрева
  if (topicName.equals(mqttGetTopic(mqttSettings, mqttHeaterMode))) {
    if (topicValue.equalsIgnoreCase(mqttHeaterDOff) || topicValue.equalsIgnoreCase(mqttHeaterSOff)) {
      heaterMode = 0;
    };
    if (topicValue.equalsIgnoreCase(mqttHeaterDOn) || topicValue.equalsIgnoreCase(mqttHeaterSOn)) {
      heaterMode = 1;
    };
    if (topicValue.equalsIgnoreCase(mqttHeaterDAuto) || topicValue.equalsIgnoreCase(mqttHeaterSAuto)) {
      heaterMode = 2;
    };
    
    EEPROM_uint8_write(addrHeaterMode, heaterMode);
    
    Serial.print(F("SETTINGS :: set value 'heaterMode' = "));
    Serial.println(heaterMode);

    if (heaterMode != 2) {
      heaterControl();
      pubHeaterState();
    };

    return true;
  };

  // Настройка границ температурного режима
  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttHeaterMin))) {
    heaterAutoOn = topicValue.toInt();
    EEPROM_uint8_write(addrHeaterAutoOn, heaterAutoOn);

    Serial.print(F("SETTINGS :: set value 'heaterAutoOn' = "));
    Serial.println(heaterAutoOn);

    if (heaterMode == 2) {
      heaterControl();
      pubHeaterState();
    };

    return true;
  };
  
  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttHeaterMax))) {
    heaterAutoOff = topicValue.toInt();
    EEPROM_uint8_write(addrHeaterAutoOff, heaterAutoOff);
    
    Serial.print(F("SETTINGS :: set value 'heaterAutoOff' = "));
    Serial.println(heaterAutoOff);

    if (heaterMode == 2) {
      heaterControl();
      pubHeaterState();
    };

    return true;
  };

  // Корректировка показаний датчиков
  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttSensor1, mqttTemperature))) {
    sensCorrTemp1 = atof(topicValue.c_str());
    EEPROM_float_write(addrCorrTemp1, sensCorrTemp1);
    
    Serial.print(F("SETTINGS :: set value 'sensCorrTemp1' = "));
    Serial.println(sensCorrTemp1);

    return true;
  };

  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttSensor1, mqttPressure))) {
    sensCorrPress1 = atof(topicValue.c_str());
    EEPROM_float_write(addrCorrPress1, sensCorrPress1);
    
    Serial.print(F("SETTINGS :: set value 'sensCorrPress1' = "));
    Serial.println(sensCorrPress1);

    return true;
  };

  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttSensor1, mqttHumidity))) {
    sensCorrHum1 = atof(topicValue.c_str());
    EEPROM_float_write(addrCorrHum1, sensCorrHum1);
    
    Serial.print(F("SETTINGS :: set value 'sensCorrHum1' = "));
    Serial.println(sensCorrHum1);

    return true;
  };

  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttSensor2, mqttTemperature))) {
    sensCorrTemp2 = atof(topicValue.c_str());
    EEPROM_float_write(addrCorrTemp2, sensCorrTemp2);
    
    Serial.print(F("SETTINGS :: set value 'sensCorrTemp2' = "));
    Serial.println(sensCorrTemp2);

    return true;
  };

  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttSensor2, mqttHumidity))) {
    sensCorrHum2 = atof(topicValue.c_str());
    EEPROM_float_write(addrCorrHum2, sensCorrHum2);
    
    Serial.print(F("SETTINGS :: set value 'sensCorrHum2' = "));
    Serial.println(sensCorrHum2);

    return true;
  };

  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttSensor3, mqttTemperature))) {
    sensCorrTemp3 = atof(topicValue.c_str());
    EEPROM_float_write(addrCorrTemp3, sensCorrTemp3);
    
    Serial.print(F("SETTINGS :: set value 'sensCorrTemp3' = "));
    Serial.println(sensCorrTemp3);

    return true;
  };

  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttSensor3, mqttHumidity))) {
    sensCorrHum3 = atof(topicValue.c_str());
    EEPROM_float_write(addrCorrHum3, sensCorrHum3);
    
    Serial.print(F("SETTINGS :: set value 'sensCorrHum3' = "));
    Serial.println(sensCorrHum3);

    return true;
  };

  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttSensor4, mqttTemperature))) {
    sensCorrTemp4 = atof(topicValue.c_str());
    EEPROM_float_write(addrCorrTemp4, sensCorrTemp4);
    
    Serial.print(F("SETTINGS :: set value 'sensCorrTemp4' = "));
    Serial.println(sensCorrTemp4);

    return true;
  };

  if (topicName.equalsIgnoreCase(mqttGetTopic(mqttSettings, mqttSensor4, mqttHumidity))) {
    sensCorrHum4 = atof(topicValue.c_str());
    EEPROM_float_write(addrCorrHum4, sensCorrHum4);
    
    Serial.print(F("SETTINGS :: set value 'sensCorrHum4' = "));
    Serial.println(sensCorrHum4);
    
    return true;
  };

  return false;
}
